@@ -12,11 +12,10 @@
 
 #include <systemstats/SensorContainer.h>
 
-#ifdef HAVE_SENSORS
 #include <sensors/sensors.h>
-#endif
 
 #include "linuxcpu.h"
+#include "loadaverages.h"
 
 struct CpuInfo
 {
@@ -29,6 +28,8 @@ struct CpuInfo
 LinuxCpuPluginPrivate::LinuxCpuPluginPrivate(CpuPlugin *q)
     : CpuPluginPrivate(q)
 {
+    m_loadAverages = new LoadAverages(m_container);
+
     // Parse /proc/cpuinfo for information about cpus
     QFile cpuinfo("/proc/cpuinfo");
     cpuinfo.open(QIODevice::ReadOnly);
@@ -73,23 +74,26 @@ LinuxCpuPluginPrivate::LinuxCpuPluginPrivate(CpuPlugin *q)
                              : i18nc("@title", "Core %1", ++numCores[entry.cpu]);
 
         auto cpu = new LinuxCpuObject(QStringLiteral("cpu%1").arg(entry.id), name, m_container);
-        cpu->initialize(entry.frequency);
         m_cpus.push_back(cpu);
         m_cpusBySystemIds.insert({entry.cpu, entry.core}, cpu);
     }
 
+    addSensors();
+    for (int i = 0; i < m_cpus.size(); ++i) {
+        m_cpus.at(i)->initialize(cpus.at(i).frequency);
+    }
     m_allCpus = new LinuxAllCpusObject(m_container);
     m_allCpus->initialize();
     m_allCpus->setCounts(cpuCount, m_cpus.size());
 
-    addSensors();
 }
 
 void LinuxCpuPluginPrivate::update()
 {
+    m_loadAverages->update();
+
     auto isSubscribed = [] (const KSysGuard::SensorObject *o) {return o->isSubscribed();};
-    const auto objects = m_container->objects();
-    if (std::none_of(objects.cbegin(), objects.cend(), isSubscribed)) {
+    if (std::none_of(m_cpusBySystemIds.cbegin(), m_cpusBySystemIds.cend(), isSubscribed) && !m_allCpus->isSubscribed()) {
         return;
     }
 
@@ -126,7 +130,6 @@ void LinuxCpuPluginPrivate::update()
 
 void LinuxCpuPluginPrivate::addSensors()
 {
-#ifdef HAVE_SENSORS
     int number = 0;
     while (const sensors_chip_name * const chipName = sensors_get_detected_chips(nullptr, &number)) {
         char name[100];
@@ -137,13 +140,11 @@ void LinuxCpuPluginPrivate::addSensors()
             addSensorsAmd(chipName);
         }
     }
-#endif
 }
 
 // Documentation: https://www.kernel.org/doc/html/latest/hwmon/coretemp.html
 void LinuxCpuPluginPrivate::addSensorsIntel(const sensors_chip_name * const chipName)
 {
-#ifdef HAVE_SENSORS
     int featureNumber = 0;
     QHash<unsigned int,  sensors_feature const *> coreFeatures;
     int physicalId = -1;
@@ -170,16 +171,14 @@ void LinuxCpuPluginPrivate::addSensorsIntel(const sensors_chip_name * const chip
             // Naturally they share the same temperature sensor and have the same coreId.
             auto cpu_range = m_cpusBySystemIds.equal_range({physicalId, feature.key()});
             for (auto cpu_it = cpu_range.first; cpu_it != cpu_range.second; ++cpu_it) {
-                (*cpu_it)->temperatureSensor()->setFeature(chipName, feature.value());
+                (*cpu_it)->makeTemperatureSensor(chipName, feature.value());
             }
         }
     }
-#endif
 }
 
 void LinuxCpuPluginPrivate::addSensorsAmd(const sensors_chip_name * const chipName)
 {
-#ifdef HAVE_SENSORS
     // All Processors should have the Tctl pseudo temperature as temp1. Newer ones have the real die
     // temperature Tdie as temp2. Some of those have temperatures for each core complex die (CCD) as
     // temp3-6 or temp3-10 depending on the number of CCDS.
@@ -211,7 +210,7 @@ void LinuxCpuPluginPrivate::addSensorsAmd(const sensors_chip_name * const chipNa
 
     auto setSingleSensor = [this, chipName] (const sensors_feature * const feature) {
         for (auto &cpu : m_cpusBySystemIds) {
-            cpu->temperatureSensor()->setFeature(chipName, feature);
+            cpu->makeTemperatureSensor(chipName, feature);
         }
     };
     if (tdie) {
@@ -219,5 +218,4 @@ void LinuxCpuPluginPrivate::addSensorsAmd(const sensors_chip_name * const chipNa
     } else if (tctl) {
         setSingleSensor(tctl);
     }
-#endif
 }
